using HackathonPreparing.ApiService.GlobalHelpers;
using HackathonPreparing.ApiService.UserSlice.EfCore;
using HackathonPreparing.ApiService.UserSlice.EfCore.Models;
using HackathonPreparing.ApiService.UserSlice.OpenApi;
using HackathonPreparing.ApiService.WeatherForecastSlice.EfCore;
using HackathonPreparing.ApiService.WeatherForecastSlice.Endpoints;
using Microsoft.AspNetCore.Identity;
using Microsoft.OpenApi.Models;

var builder = WebApplication.CreateBuilder(args);

builder.Services.AddEndpointsApiExplorer();
builder.Services.AddSwaggerGen(c =>
{
    c.SchemaFilter<UserOpenApiSchemeFilter>();
    c.AddSecurityDefinition("AccessToken", new OpenApiSecurityScheme {
        In = ParameterLocation.Header,
        Description = "can get in /login or /register",
        Name = "Authorization",
        Type = SecuritySchemeType.Http,
        BearerFormat = "JWT",
        Scheme = "bearer"
    });

    c.AddSecurityRequirement(new OpenApiSecurityRequirement{
        {
            new OpenApiSecurityScheme {
                Reference = new OpenApiReference{
                    Type = ReferenceType.SecurityScheme,
                    Id = "AccessToken"
                }
            },
            new string[]{}
        }
    });
});

// Add service defaults & Aspire components.
//we dont need it now
//builder.AddServiceDefaults();



builder.Services.AddAuthorization();

builder.Services.AddAuthorizationBuilder()
    .AddPolicy("moderator", policy =>
        policy.RequireRole("MODERATOR"));

//TODO BEARER
builder.Services.AddAuthentication().AddBearerToken(IdentityConstants.BearerScheme);

builder.Services.AddIdentity<User, IdentityRole<int>>()
    .AddEntityFrameworkStores<UserContext>()
    .AddApiEndpoints();

// Add services to the container.
builder.Services.AddProblemDetails();

builder.AddDbContextWithDefaultConfiguration<WeatherForecastContext>();
builder.AddDbContextWithDefaultConfiguration<UserContext>();

var app = builder.Build();

app.UsePathBase(new PathString("/api"));

if (app.Environment.IsDevelopment())
{
    var logger = app.Services.GetRequiredService<ILogger<Program>>();
    logger.LogWarning("!!!!!!! Call /ensure-created enpoint or /migrate if no db !!!!!!!!");
    app.UseSwagger();
    app.UseSwaggerUI();
    app.AddDbHelpersEndpoints();
}

// Configure the HTTP request pipeline.
app.UseExceptionHandler();
app.UseAuthorization();
app.UseAuthentication();

app.AddWeatherForecastEndpoints();

app.MapDefaultEndpoints();

app.MapGroup("/user")
    .MapIdentityApi<User>()
    .WithTags("user");

app.Run();
